@@ -27,6 +27,9 @@
 #include <QStack>
 #include <QMutex>
 #include <QMutexLocker>
+#include <QReadWriteLock>
+#include <QReadLocker>
+#include <QWriteLocker>
 
 #include <KoIcon.h>
 #include <kis_icon.h>
@@ -55,54 +58,10 @@
 #include "krita_utils.h"
 #include "kis_layer_properties_icons.h"
 #include "kis_layer_utils.h"
+#include "kis_projection_leaf.h"
+#include "KisSafeNodeProjectionStore.h"
 
 
-class KisSafeProjection {
-public:
-    KisPaintDeviceSP getDeviceLazy(KisPaintDeviceSP prototype) {
-        QMutexLocker locker(&m_lock);
-
-        if (!m_reusablePaintDevice) {
-            m_reusablePaintDevice = new KisPaintDevice(*prototype);
-        }
-        if(!m_projection ||
-           *m_projection->colorSpace() != *prototype->colorSpace()) {
-            m_projection = m_reusablePaintDevice;
-            m_projection->makeCloneFromRough(prototype, prototype->extent());
-            m_projection->setProjectionDevice(true);
-        }
-
-        return m_projection;
-    }
-
-    void tryCopyFrom(const KisSafeProjection &rhs) {
-        QMutexLocker locker(&m_lock);
-
-        if (!rhs.m_projection) return;
-
-        if (!m_reusablePaintDevice) {
-            m_reusablePaintDevice = new KisPaintDevice(*rhs.m_projection);
-            m_projection = m_reusablePaintDevice;
-        } else {
-            m_projection = m_reusablePaintDevice;
-            m_projection->makeCloneFromRough(rhs.m_projection, rhs.m_projection->extent());
-        }
-    }
-
-    void freeDevice() {
-        QMutexLocker locker(&m_lock);
-        m_projection = 0;
-        if(m_reusablePaintDevice) {
-            m_reusablePaintDevice->clear();
-        }
-    }
-
-private:
-    QMutex m_lock;
-    KisPaintDeviceSP m_projection;
-    KisPaintDeviceSP m_reusablePaintDevice;
-};
-
 class KisCloneLayersList {
 public:
     void addClone(KisCloneLayerWSP cloneLayer) {
@@ -133,49 +92,129 @@ private:
     QList<KisCloneLayerWSP> m_clonesList;
 };
 
+class KisLayerMasksCache {
+public:
+    KisLayerMasksCache(KisLayer *parent)
+        : m_parent(parent)
+    {
+    }
+
+    KisSelectionMaskSP selectionMask() {
+        QReadLocker readLock(&m_lock);
+
+        if (!m_isSelectionMaskValid) {
+            readLock.unlock();
+
+            QWriteLocker writeLock(&m_lock);
+            if (!m_isSelectionMaskValid) {
+                KoProperties properties;
+                properties.setProperty("active", true);
+                properties.setProperty("visible", true);
+                QList<KisNodeSP> masks = m_parent->childNodes(QStringList("KisSelectionMask"), properties);
+
+                // return the first visible mask
+                Q_FOREACH (KisNodeSP mask, masks) {
+                    if (mask) {
+                        m_selectionMask = dynamic_cast<KisSelectionMask*>(mask.data());
+                        break;
+                    }
+                }
+                m_isSelectionMaskValid = true;
+            }
+
+            // return under write lock
+            return m_selectionMask;
+        }
+
+        // return under read lock
+        return m_selectionMask;
+    }
+
+    QList<KisEffectMaskSP> effectMasks() {
+        QReadLocker readLock(&m_lock);
+
+        if (!m_isEffectMasksValid) {
+            readLock.unlock();
+
+            QWriteLocker writeLock(&m_lock);
+            if (!m_isEffectMasksValid) {
+                m_effectMasks = m_parent->searchEffectMasks(0);
+                m_isEffectMasksValid = true;
+            }
+
+            // return under write lock
+            return m_effectMasks;
+        }
+
+        // return under read lock
+        return m_effectMasks;
+    }
+
+    void setDirty()
+    {
+        QWriteLocker l(&m_lock);
+        m_isSelectionMaskValid = false;
+        m_isEffectMasksValid = false;
+        m_selectionMask = 0;
+        m_effectMasks.clear();
+    }
+
+private:
+    KisLayer *m_parent;
+
+    QReadWriteLock m_lock;
+
+    bool m_isSelectionMaskValid = false;
+    bool m_isEffectMasksValid = false;
+    KisSelectionMaskSP m_selectionMask;
+    QList<KisEffectMaskSP> m_effectMasks;
+};
+
 struct Q_DECL_HIDDEN KisLayer::Private
 {
-    KisImageWSP image;
+    Private(KisLayer *q)
+        : masksCache(q)
+    {
+    }
+
     QBitArray channelFlags;
     KisMetaData::Store* metaDataStore;
-    KisSafeProjection safeProjection;
     KisCloneLayersList clonesList;
 
     KisPSDLayerStyleSP layerStyle;
     KisLayerStyleProjectionPlaneSP layerStyleProjectionPlane;
 
     KisAbstractProjectionPlaneSP projectionPlane;
+    KisSafeNodeProjectionStoreSP safeProjection;
 
-    KisSelectionMaskSP selectionMask;
-    QList<KisEffectMaskSP> effectMasks;
+    KisLayerMasksCache masksCache;
 };
 
 
 KisLayer::KisLayer(KisImageWSP image, const QString &name, quint8 opacity)
-        : KisNode()
-        , m_d(new Private)
+        : KisNode(image)
+        , m_d(new Private(this))
 {
     setName(name);
     setOpacity(opacity);
-    m_d->image = image;
     m_d->metaDataStore = new KisMetaData::Store();
     m_d->projectionPlane = toQShared(new KisLayerProjectionPlane(this));
-    notifyChildMaskChanged(KisNodeSP());
+    m_d->safeProjection = new KisSafeNodeProjectionStore();
+    m_d->safeProjection->setImage(image);
 }
 
 KisLayer::KisLayer(const KisLayer& rhs)
         : KisNode(rhs)
-        , m_d(new Private())
+        , m_d(new Private(this))
 {
     if (this != &rhs) {
-        m_d->image = rhs.m_d->image;
         m_d->metaDataStore = new KisMetaData::Store(*rhs.m_d->metaDataStore);
         m_d->channelFlags = rhs.m_d->channelFlags;
 
-        m_d->safeProjection.tryCopyFrom(rhs.m_d->safeProjection);
-
         setName(rhs.name());
         m_d->projectionPlane = toQShared(new KisLayerProjectionPlane(this));
+        m_d->safeProjection = new KisSafeNodeProjectionStore(*rhs.m_d->safeProjection);
+        m_d->safeProjection->setImage(image());
 
         if (rhs.m_d->layerStyle) {
             m_d->layerStyle = rhs.m_d->layerStyle->clone();
@@ -187,7 +226,6 @@ KisLayer::KisLayer(const KisLayer& rhs)
                                                      m_d->layerStyle));
             }
         }
-        notifyChildMaskChanged(KisNodeSP());
     }
 }
 
@@ -199,7 +237,7 @@ KisLayer::~KisLayer()
 
 const KoColorSpace * KisLayer::colorSpace() const
 {
-    KisImageSP image = m_d->image.toStrongRef();
+    KisImageSP image = this->image();
     if (!image) {
         return nullptr;
     }
@@ -344,30 +382,16 @@ void KisLayer::setTemporary(bool t)
     setNodeProperty("temporary", t);
 }
 
-KisImageWSP KisLayer::image() const
-{
-    return m_d->image;
-}
-
 void KisLayer::setImage(KisImageWSP image)
 {
-    m_d->image = image;
-
     // we own the projection device, so we should take care about it
     KisPaintDeviceSP projection = this->projection();
     if (projection && projection != original()) {
         projection->setDefaultBounds(new KisDefaultBounds(image));
     }
+    m_d->safeProjection->setImage(image);
 
-    KisNodeSP node = firstChild();
-    while (node) {
-        KisLayerUtils::recursiveApplyNodes(node,
-                                           [image] (KisNodeSP node) {
-                                               node->setImage(image);
-                                           });
-
-        node = node->nextSibling();
-    }
+    KisNode::setImage(image);
 }
 
 bool KisLayer::canMergeAndKeepBlendOptions(KisLayerSP otherLayer)
@@ -469,34 +493,14 @@ void KisLayer::updateClones(const QRect &rect)
     m_d->clonesList.setDirty(rect);
 }
 
-void KisLayer::notifyChildMaskChanged(KisNodeSP changedChildMask)
+void KisLayer::notifyChildMaskChanged()
 {
-    Q_UNUSED(changedChildMask);
-
-    updateSelectionMask();
-    updateEffectMasks();
+    m_d->masksCache.setDirty();
 }
 
 KisSelectionMaskSP KisLayer::selectionMask() const
 {
-    return m_d->selectionMask;
-}
-
-void KisLayer::updateSelectionMask()
-{
-    KoProperties properties;
-    properties.setProperty("active", true);
-    properties.setProperty("visible", true);
-    QList<KisNodeSP> masks = childNodes(QStringList("KisSelectionMask"), properties);
-
-    // return the first visible mask
-    Q_FOREACH (KisNodeSP mask, masks) {
-        if (mask) {
-            m_d->selectionMask = dynamic_cast<KisSelectionMask*>(mask.data());
-            return;
-        }
-    }
-    m_d->selectionMask = KisSelectionMaskSP();
+    return m_d->masksCache.selectionMask();
 }
 
 KisSelectionSP KisLayer::selection() const
@@ -507,7 +511,7 @@ KisSelectionSP KisLayer::selection() const
         return mask->selection();
     }
 
-    KisImageSP image = m_d->image.toStrongRef();
+    KisImageSP image = this->image();
     if (image) {
         return image->globalSelection();
     }
@@ -517,9 +521,9 @@ KisSelectionSP KisLayer::selection() const
 ///////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////
 
-const QList<KisEffectMaskSP> &KisLayer::effectMasks() const
+QList<KisEffectMaskSP> KisLayer::effectMasks() const
 {
-    return m_d->effectMasks;
+    return m_d->masksCache.effectMasks();
 }
 
 QList<KisEffectMaskSP> KisLayer::effectMasks(KisNodeSP lastNode) const
@@ -532,27 +536,27 @@ QList<KisEffectMaskSP> KisLayer::effectMasks(KisNodeSP lastNode) const
     }
 }
 
-void KisLayer::updateEffectMasks()
-{
-    m_d->effectMasks = searchEffectMasks(KisNodeSP());
-}
-
 QList<KisEffectMaskSP> KisLayer::searchEffectMasks(KisNodeSP lastNode) const
 {
     QList<KisEffectMaskSP> masks;
 
-    if (childCount() > 0) {
-        KoProperties properties;
-        properties.setProperty("visible", true);
-        QList<KisNodeSP> nodes = childNodes(QStringList("KisEffectMask"), properties);
+    KIS_SAFE_ASSERT_RECOVER_NOOP(projectionLeaf());
 
-        Q_FOREACH (const KisNodeSP& node, nodes) {
-            if (node == lastNode) break;
+    KisProjectionLeafSP child = projectionLeaf()->firstChild();
+    while (child) {
+        if (child->node() == lastNode) break;
 
-            KisEffectMaskSP mask = dynamic_cast<KisEffectMask*>(const_cast<KisNode*>(node.data()));
-            if (mask)
+        KIS_SAFE_ASSERT_RECOVER_NOOP(child);
+        KIS_SAFE_ASSERT_RECOVER_NOOP(child->node());
+
+        if (child->visible()) {
+            KisEffectMaskSP mask = dynamic_cast<KisEffectMask*>(const_cast<KisNode*>(child->node().data()));
+            if (mask) {
                 masks.append(mask);
+            }
         }
+
+        child = child->nextSibling();
     }
 
     return masks;
@@ -560,7 +564,7 @@ QList<KisEffectMaskSP> KisLayer::searchEffectMasks(KisNodeSP lastNode) const
 
 bool KisLayer::hasEffectMasks() const
 {
-    return  !m_d->effectMasks.empty();
+    return  !m_d->masksCache.effectMasks().isEmpty();
 }
 
 QRect KisLayer::masksChangeRect(const QList<KisEffectMaskSP> &masks,
@@ -734,13 +738,11 @@ QRect KisLayer::updateProjection(const QRect& rect, KisNodeSP filthyNode)
             !originalDevice) return QRect();
 
     if (!needProjection() && !hasEffectMasks()) {
-        m_d->safeProjection.freeDevice();
+        m_d->safeProjection->releaseDevice();
     } else {
 
         if (!updatedRect.isEmpty()) {
-            KisPaintDeviceSP projection =
-                m_d->safeProjection.getDeviceLazy(originalDevice);
-
+            KisPaintDeviceSP projection = m_d->safeProjection->getDeviceLazy(originalDevice);
             updatedRect = applyMasks(originalDevice, projection,
                                      updatedRect, filthyNode, 0);
         }
@@ -804,7 +806,7 @@ KisPaintDeviceSP KisLayer::projection() const
     KisPaintDeviceSP originalDevice = original();
 
     return needProjection() || hasEffectMasks() ?
-        m_d->safeProjection.getDeviceLazy(originalDevice) : originalDevice;
+        m_d->safeProjection->getDeviceLazy(originalDevice) : originalDevice;
 }
 
 QRect KisLayer::changeRect(const QRect &rect, PositionToFilthy pos) const
@@ -847,7 +849,7 @@ QRect KisLayer::changeRect(const QRect &rect, PositionToFilthy pos) const
 void KisLayer::childNodeChanged(KisNodeSP changedChildNode)
 {
     if (dynamic_cast<KisMask*>(changedChildNode.data())) {
-        notifyChildMaskChanged(changedChildNode);
+        notifyChildMaskChanged();
     }
 }
 
@@ -861,6 +863,23 @@ QRect KisLayer::outgoingChangeRect(const QRect &rect) const
     return rect;
 }
 
+QRect KisLayer::needRectForOriginal(const QRect &rect) const
+{
+    QRect needRect = rect;
+
+    const QList<KisEffectMaskSP> masks = effectMasks();
+
+    if (!masks.isEmpty()) {
+        QStack<QRect> applyRects;
+        bool needRectVaries;
+
+        needRect = masksNeedRect(masks, rect,
+                                 applyRects, needRectVaries);
+    }
+
+    return needRect;
+}
+
 QImage KisLayer::createThumbnail(qint32 w, qint32 h)
 {
     if (w == 0 || h == 0) {
