@@ -1,4 +1,4 @@
-﻿/* This file is part of the Krita project
+/* This file is part of the Krita project
  *
  * Copyright (C) 2014 Boudewijn Rempt <boud@valdyas.org>
  *
@@ -47,11 +47,13 @@
 #include <KoStoreDevice.h>
 #include <KoDialog.h>
 
+#include <KisUsageLogger.h>
 #include <klocalizedstring.h>
 #include <kis_debug.h>
+#include <kis_generator_layer.h>
+#include <kis_generator_registry.h>
 #include <kdesktopfile.h>
 #include <kconfiggroup.h>
-#include <QTemporaryFile>
 #include <kbackup.h>
 
 #include <QTextBrowser>
@@ -76,6 +78,7 @@
 #include <QFutureWatcher>
 
 // Krita Image
+#include <kis_image_animation_interface.h>
 #include <kis_config.h>
 #include <flake/kis_shape_layer.h>
 #include <kis_group_layer.h>
@@ -118,6 +121,8 @@
 #include "kis_async_action_feedback.h"
 #include "KisCloneDocumentStroke.h"
 
+#include <KisMirrorAxisConfig.h>
+
 
 // Define the protocol used here for embedded documents' URL
 // This used to "store" but QUrl didn't like it,
@@ -233,7 +238,7 @@ public:
         , lastMod(firstMod)
         , nserver(new KisNameServer(1))
         , imageIdleWatcher(2000 /*ms*/)
-        , globalAssistantsColor(KisConfig().defaultAssistantsColor())
+        , globalAssistantsColor(KisConfig(true).defaultAssistantsColor())
         , savingLock(&savingMutex)
         , batchMode(false)
     {
@@ -253,6 +258,7 @@ public:
         , autoSaveTimer(new QTimer(q))
         , undoStack(new UndoStack(q))
         , guidesConfig(rhs.guidesConfig)
+        , mirrorAxisConfig(rhs.mirrorAxisConfig)
         , m_bAutoDetectedMime(rhs.m_bAutoDetectedMime)
         , m_url(rhs.m_url)
         , m_file(rhs.m_file)
@@ -265,6 +271,7 @@ public:
         , imageIdleWatcher(2000 /*ms*/)
         , assistants(rhs.assistants) // WARNING: assistants should not store pointers to the document!
         , globalAssistantsColor(rhs.globalAssistantsColor)
+        , paletteList(rhs.paletteList)
         , gridConfig(rhs.gridConfig)
         , savingLock(&savingMutex)
         , batchMode(rhs.batchMode)
@@ -298,6 +305,7 @@ public:
     KUndo2Stack *undoStack = 0;
 
     KisGuidesConfig guidesConfig;
+    KisMirrorAxisConfig mirrorAxisConfig;
 
     bool m_bAutoDetectedMime = false; // whether the mimetype in the arguments was detected by the part itself
     QUrl m_url; // local url - the one displayed to the user.
@@ -328,6 +336,8 @@ public:
 
     KisSharedPtr<KisReferenceImagesLayer> referenceImagesLayer;
 
+    QList<KoColorSet*> paletteList;
+
     KisGridConfig gridConfig;
 
     StdLockableWrapper<QMutex> savingLock;
@@ -388,11 +398,11 @@ public:
     }
 
     ~StrippedSafeSavingLocker() {
-         if (m_locked) {
-             m_imageLock.unlock();
-             m_savingLock.unlock();
-         }
-     }
+        if (m_locked) {
+            m_imageLock.unlock();
+            m_savingLock.unlock();
+        }
+    }
 
     bool successfullyLocked() const {
         return m_locked;
@@ -417,9 +427,9 @@ KisDocument::KisDocument()
     KisResourceServerProvider::instance();
 
     d->shapeController = new KisShapeController(this, d->nserver),
-    d->koShapeController = new KoShapeController(0, d->shapeController),
+            d->koShapeController = new KoShapeController(0, d->shapeController),
 
-    slotConfigChanged();
+            slotConfigChanged();
 }
 
 KisDocument::KisDocument(const KisDocument &rhs)
@@ -432,9 +442,9 @@ KisDocument::KisDocument(const KisDocument &rhs)
     setObjectName(rhs.objectName());
 
     d->shapeController = new KisShapeController(this, d->nserver),
-    d->koShapeController = new KoShapeController(0, d->shapeController),
+            d->koShapeController = new KoShapeController(0, d->shapeController),
 
-    slotConfigChanged();
+            slotConfigChanged();
 
     // clone the image with keeping the GUIDs of the layers intact
     // NOTE: we expect the image to be locked!
@@ -526,22 +536,45 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
         return false;
     }
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     if (cfg.backupFile() && filePathInfo.exists()) {
-        KBackup::backupFile(job.filePath);
+
+        QString backupDir;
+
+        switch(cfg.readEntry<int>("backupfilelocation", 0)) {
+        case 1:
+            backupDir = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
+            break;
+        case 2:
+            backupDir = QStandardPaths::writableLocation(QStandardPaths::TempLocation);
+            break;
+        default:
+            // Do nothing: the empty string is user file location
+            break;
+        }
+
+        int numOfBackupsKept = cfg.readEntry<int>("numberofbackupfiles", 1);
+        QString suffix = cfg.readEntry<QString>("backupfilesuffix", "~");
+
+        if (numOfBackupsKept == 1) {
+            KBackup::simpleBackupFile(job.filePath, backupDir, suffix);
+        }
+        else if (numOfBackupsKept > 2) {
+            KBackup::numberedBackupFile(job.filePath, backupDir, suffix, numOfBackupsKept);
+        }
     }
 
     KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!job.mimeType.isEmpty(), false);
 
     const QString actionName =
-        job.flags & KritaUtils::SaveIsExporting ?
-        i18n("Exporting Document...") :
-        i18n("Saving Document...");
+            job.flags & KritaUtils::SaveIsExporting ?
+                i18n("Exporting Document...") :
+                i18n("Saving Document...");
 
     bool started =
-        initiateSavingInBackground(actionName,
-                                   this, SLOT(slotCompleteSavingDocument(KritaUtils::ExportFileJob, KisImportExportFilter::ConversionStatus,QString)),
-                                   job, exportConfiguration);
+            initiateSavingInBackground(actionName,
+                                       this, SLOT(slotCompleteSavingDocument(KritaUtils::ExportFileJob,KisImportExportFilter::ConversionStatus,QString)),
+                                       job, exportConfiguration);
 
     if (!started) {
         emit canceled(QString());
@@ -559,6 +592,16 @@ bool KisDocument::exportDocument(const QUrl &url, const QByteArray &mimeType, bo
         flags |= SaveShowWarnings;
     }
 
+    KisUsageLogger::log(QString("Exporting Document: %1 as %2. %3 * %4 pixels, %5 layers, %6 frames, %7 framerate. Export configuration: %8")
+                        .arg(url.toLocalFile())
+                        .arg(QString::fromLatin1(mimeType))
+                        .arg(d->image->width())
+                        .arg(d->image->height())
+                        .arg(d->image->nlayers())
+                        .arg(d->image->animationInterface()->totalLength())
+                        .arg(d->image->animationInterface()->framerate())
+                        .arg(exportConfiguration ? exportConfiguration->toXML() : "No configuration"));
+
     return exportDocumentImpl(KritaUtils::ExportFileJob(url.toLocalFile(),
                                                         mimeType,
                                                         flags),
@@ -566,11 +609,23 @@ bool KisDocument::exportDocument(const QUrl &url, const QByteArray &mimeType, bo
 
 }
 
-bool KisDocument::saveAs(const QUrl &url, const QByteArray &mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
+bool KisDocument::saveAs(const QUrl &_url, const QByteArray &mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
 {
     using namespace KritaUtils;
 
-    return exportDocumentImpl(ExportFileJob(url.toLocalFile(),
+    KisUsageLogger::log(QString("Saving Document %9 as %1 (mime: %2). %3 * %4 pixels, %5 layers.  %6 frames, %7 framerate. Export configuration: %8")
+                        .arg(_url.toLocalFile())
+                        .arg(QString::fromLatin1(mimeType))
+                        .arg(d->image->width())
+                        .arg(d->image->height())
+                        .arg(d->image->nlayers())
+                        .arg(d->image->animationInterface()->totalLength())
+                        .arg(d->image->animationInterface()->framerate())
+                        .arg(exportConfiguration ? exportConfiguration->toXML() : "No configuration")
+                        .arg(url().toLocalFile()));
+
+
+    return exportDocumentImpl(ExportFileJob(_url.toLocalFile(),
                                             mimeType,
                                             showWarnings ? SaveShowWarnings : SaveNone),
                               exportConfiguration);
@@ -623,16 +678,29 @@ void KisDocument::slotCompleteSavingDocument(const KritaUtils::ExportFileJob &jo
         }
     } else {
         if (!(job.flags & KritaUtils::SaveIsExporting)) {
+            const QString existingAutoSaveBaseName = localFilePath();
+            const bool wasRecovered = isRecovered();
+
             setUrl(QUrl::fromLocalFile(job.filePath));
             setLocalFilePath(job.filePath);
             setMimeType(job.mimeType);
             updateEditingTime(true);
 
             if (!d->modifiedWhileSaving) {
-                d->undoStack->setClean();
+                /**
+                 * If undo stack is already clean/empty, it doesn't emit any
+                 * signals, so we might forget update document modified state
+                 * (which was set, e.g. while recovering an autosave file)
+                 */
+
+                if (d->undoStack->isClean()) {
+                    setModified(false);
+                } else {
+                    d->undoStack->setClean();
+                }
             }
             setRecovered(false);
-            removeAutoSaveFiles();
+            removeAutoSaveFiles(existingAutoSaveBaseName, wasRecovered);
         }
 
         emit completed();
@@ -687,9 +755,18 @@ KisDocument* KisDocument::lockAndCloneForSaving()
 
 bool KisDocument::exportDocumentSync(const QUrl &url, const QByteArray &mimeType, KisPropertiesConfigurationSP exportConfiguration)
 {
-    Private::StrippedSafeSavingLocker locker(&d->savingMutex, d->image);
-    if (!locker.successfullyLocked()) {
-        return false;
+    {
+
+        /**
+         * The caller guarantees that noone else uses the document (usually,
+         * it is a temporary docuent created specifically for exporting), so
+         * we don't need to copy or lock the document. Instead we should just
+         * ensure the barrier lock is synced and then released.
+         */
+        Private::StrippedSafeSavingLocker locker(&d->savingMutex, d->image);
+        if (!locker.successfullyLocked()) {
+            return false;
+        }
     }
 
     d->savingImage = d->image;
@@ -697,7 +774,7 @@ bool KisDocument::exportDocumentSync(const QUrl &url, const QByteArray &mimeType
     const QString fileName = url.toLocalFile();
 
     KisImportExportFilter::ConversionStatus status =
-        d->importExportManager->
+            d->importExportManager->
             exportDocument(fileName, fileName, mimeType, false, exportConfiguration);
 
     d->savingImage = 0;
@@ -735,6 +812,27 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
         return false;
     }
 
+    auto waitForImage = [] (KisImageSP image) {
+        KisMainWindow *window = KisPart::instance()->currentMainwindow();
+        if (window) {
+            if (window->viewManager()) {
+                window->viewManager()->blockUntilOperationsFinishedForced(image);
+            }
+        }
+    };
+
+    {
+        KisNodeSP newRoot = clonedDocument->image()->root();
+        KIS_SAFE_ASSERT_RECOVER(!KisLayerUtils::hasDelayedNodeWithUpdates(newRoot)) {
+            KisLayerUtils::forceAllDelayedNodesUpdate(newRoot);
+            waitForImage(clonedDocument->image());
+        }
+    }
+
+    KIS_SAFE_ASSERT_RECOVER(clonedDocument->image()->isIdle()) {
+        waitForImage(clonedDocument->image());
+    }
+
     KIS_ASSERT_RECOVER_RETURN_VALUE(!d->backgroundSaveDocument, false);
     KIS_ASSERT_RECOVER_RETURN_VALUE(!d->backgroundSaveJob.isValid(), false);
     d->backgroundSaveDocument.reset(clonedDocument.take());
@@ -746,21 +844,21 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
     }
 
     connect(d->backgroundSaveDocument.data(),
-            SIGNAL(sigBackgroundSavingFinished(KisImportExportFilter::ConversionStatus, const QString&)),
+            SIGNAL(sigBackgroundSavingFinished(KisImportExportFilter::ConversionStatus,QString)),
             this,
-            SLOT(slotChildCompletedSavingInBackground(KisImportExportFilter::ConversionStatus, const QString&)));
+            SLOT(slotChildCompletedSavingInBackground(KisImportExportFilter::ConversionStatus,QString)));
 
 
     connect(this, SIGNAL(sigCompleteBackgroundSaving(KritaUtils::ExportFileJob,KisImportExportFilter::ConversionStatus,QString)),
             receiverObject, receiverMethod, Qt::UniqueConnection);
 
     bool started =
-        d->backgroundSaveDocument->startExportInBackground(actionName,
-                                                           job.filePath,
-                                                           job.filePath,
-                                                           job.mimeType,
-                                                           job.flags & KritaUtils::SaveShowWarnings,
-                                                           exportConfiguration);
+            d->backgroundSaveDocument->startExportInBackground(actionName,
+                                                               job.filePath,
+                                                               job.filePath,
+                                                               job.mimeType,
+                                                               job.flags & KritaUtils::SaveShowWarnings,
+                                                               exportConfiguration);
 
     if (!started) {
         // the state should have been deinitialized in slotChildCompletedSavingInBackground()
@@ -796,6 +894,11 @@ void KisDocument::slotChildCompletedSavingInBackground(KisImportExportFilter::Co
     const KritaUtils::ExportFileJob job = d->backgroundSaveJob;
     d->backgroundSaveJob = KritaUtils::ExportFileJob();
 
+    KisUsageLogger::log(QString("Completed saving %1 (mime: %2). Result: %3")
+                        .arg(job.filePath)
+                        .arg(QString::fromLatin1(job.mimeType))
+                        .arg(status != KisImportExportFilter::OK ? exportErrorToUserMessage(status, errorMessage) : "OK"));
+
     emit sigCompleteBackgroundSaving(job, status, errorMessage);
 }
 
@@ -811,7 +914,7 @@ void KisDocument::slotAutoSaveImpl(std::unique_ptr<KisDocument> &&optionalCloned
 
     if (d->image->isIdle() || hadClonedDocument) {
         started = initiateSavingInBackground(i18n("Autosaving..."),
-                                             this, SLOT(slotCompleteAutoSaving(KritaUtils::ExportFileJob, KisImportExportFilter::ConversionStatus, const QString&)),
+                                             this, SLOT(slotCompleteAutoSaving(KritaUtils::ExportFileJob,KisImportExportFilter::ConversionStatus,QString)),
                                              KritaUtils::ExportFileJob(autoSaveFileName, nativeFormatMimeType(), KritaUtils::SaveIsExporting | KritaUtils::SaveInAutosaveMode),
                                              0,
                                              std::move(optionalClonedDocument));
@@ -860,7 +963,7 @@ void KisDocument::slotCompleteAutoSaving(const KritaUtils::ExportFileJob &job, K
                                     fileName,
                                     exportErrorToUserMessage(status, errorMessage)), errorMessageTimeout);
     } else {
-        KisConfig cfg;
+        KisConfig cfg(true);
         d->autoSaveDelay = cfg.autoSaveInterval();
 
         if (!d->modifiedWhileSaving) {
@@ -893,12 +996,12 @@ bool KisDocument::startExportInBackground(const QString &actionName,
 
     KisImportExportFilter::ConversionStatus initializationStatus;
     d->childSavingFuture =
-        d->importExportManager->exportDocumentAsyc(location,
-                                                   realLocation,
-                                                   mimeType,
-                                                   initializationStatus,
-                                                   showWarnings,
-                                                   exportConfiguration);
+            d->importExportManager->exportDocumentAsyc(location,
+                                                       realLocation,
+                                                       mimeType,
+                                                       initializationStatus,
+                                                       showWarnings,
+                                                       exportConfiguration);
 
     if (initializationStatus != KisImportExportFilter::ConversionStatus::OK) {
         if (d->savingUpdater) {
@@ -927,7 +1030,7 @@ void KisDocument::finishExportInBackground()
     }
 
     KisImportExportFilter::ConversionStatus status =
-        d->childSavingFuture.result();
+            d->childSavingFuture.result();
     const QString errorMessage = this->errorMessage();
 
     d->savingImage.clear();
@@ -1016,23 +1119,25 @@ QString KisDocument::generateAutoSaveFileName(const QString & path) const
 
     // Using the extension allows to avoid relying on the mime magic when opening
     const QString extension (".kra");
-    QRegularExpression autosavePattern("^\\..+-autosave.kra$");
+    QString prefix = KisConfig(true).readEntry<bool>("autosavefileshidden") ? QString(".") : QString();
+    QRegularExpression autosavePattern1("^\\..+-autosave.kra$");
+    QRegularExpression autosavePattern2("^.+-autosave.kra$");
 
     QFileInfo fi(path);
     QString dir = fi.absolutePath();
     QString filename = fi.fileName();
 
-    if (path.isEmpty() || autosavePattern.match(filename).hasMatch()) {
+    if (path.isEmpty() || autosavePattern1.match(filename).hasMatch() || autosavePattern2.match(filename).hasMatch() || !fi.isWritable()) {
         // Never saved?
 #ifdef Q_OS_WIN
         // On Windows, use the temp location (https://bugs.kde.org/show_bug.cgi?id=314921)
-        retval = QString("%1%2.%3-%4-%5-autosave%6").arg(QDir::tempPath()).arg(QDir::separator()).arg("krita").arg(qApp->applicationPid()).arg(objectName()).arg(extension);
+        retval = QString("%1%2%7%3-%4-%5-autosave%6").arg(QDir::tempPath()).arg(QDir::separator()).arg("krita").arg(qApp->applicationPid()).arg(objectName()).arg(extension).arg(prefix);
 #else
         // On Linux, use a temp file in $HOME then. Mark it with the pid so two instances don't overwrite each other's autosave file
-        retval = QString("%1%2.%3-%4-%5-autosave%6").arg(QDir::homePath()).arg(QDir::separator()).arg("krita").arg(qApp->applicationPid()).arg(objectName()).arg(extension);
+        retval = QString("%1%2%7%3-%4-%5-autosave%6").arg(QDir::homePath()).arg(QDir::separator()).arg("krita").arg(qApp->applicationPid()).arg(objectName()).arg(extension).arg(prefix);
 #endif
     } else {
-        retval = QString("%1%2.%3-autosave%4").arg(dir).arg(QDir::separator()).arg(filename).arg(extension);
+        retval = QString("%1%2%5%3-autosave%4").arg(dir).arg(QDir::separator()).arg(filename).arg(extension).arg(prefix);
     }
 
     //qDebug() << "generateAutoSaveFileName() for path" << path << ":" << retval;
@@ -1110,11 +1215,12 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
         setRecovered(true);
     }
     else {
-        if (!(flags & DontAddToRecent)) {
-            KisPart::instance()->addRecentURLToAllMainWindows(_url);
-        }
-
         if (ret) {
+
+            if (!(flags & DontAddToRecent)) {
+                KisPart::instance()->addRecentURLToAllMainWindows(_url);
+            }
+
             // Detect readonly local-files; remote files are assumed to be writable
             QFileInfo fi(url.toLocalFile());
             setReadWrite(fi.isWritable());
@@ -1196,8 +1302,9 @@ bool KisDocument::openFile()
     dbgUI << localFilePath() << "type:" << typeName;
 
     KisMainWindow *window = KisPart::instance()->currentMainwindow();
+    KoUpdaterPtr updater;
     if (window && window->viewManager()) {
-        KoUpdaterPtr updater = window->viewManager()->createUnthreadedUpdater(i18n("Opening document"));
+        updater = window->viewManager()->createUnthreadedUpdater(i18n("Opening document"));
         d->importExportManager->setUpdater(updater);
     }
 
@@ -1206,6 +1313,9 @@ bool KisDocument::openFile()
     status = d->importExportManager->importDocument(localFilePath(), typeName);
 
     if (status != KisImportExportFilter::OK) {
+        if (window && window->viewManager()) {
+            updater->cancel();
+        }
         QString msg = KisImportExportFilter::conversionStatusString(status);
         if (!msg.isEmpty()) {
             DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
@@ -1389,22 +1499,39 @@ QString KisDocument::warningMessage() const
 }
 
 
-void KisDocument::removeAutoSaveFiles()
+void KisDocument::removeAutoSaveFiles(const QString &autosaveBaseName, bool wasRecovered)
 {
     //qDebug() << "removeAutoSaveFiles";
     // Eliminate any auto-save file
-    QString asf = generateAutoSaveFileName(localFilePath());   // the one in the current dir
+    QString asf = generateAutoSaveFileName(autosaveBaseName);   // the one in the current dir
+
     //qDebug() << "\tfilename:" << asf << "exists:" << QFile::exists(asf);
     if (QFile::exists(asf)) {
         //qDebug() << "\tremoving autosavefile" << asf;
         QFile::remove(asf);
     }
     asf = generateAutoSaveFileName(QString());   // and the one in $HOME
+
     //qDebug() << "Autsavefile in $home" << asf;
     if (QFile::exists(asf)) {
         //qDebug() << "\tremoving autsavefile 2" << asf;
         QFile::remove(asf);
     }
+
+    QList<QRegularExpression> expressions;
+
+    expressions << QRegularExpression("^\\..+-autosave.kra$")
+                << QRegularExpression("^.+-autosave.kra$");
+
+    Q_FOREACH(const QRegularExpression &rex, expressions) {
+        if (wasRecovered &&
+                !autosaveBaseName.isEmpty() &&
+                rex.match(QFileInfo(autosaveBaseName).fileName()).hasMatch() &&
+                QFile::exists(autosaveBaseName)) {
+
+            QFile::remove(autosaveBaseName);
+        }
+    }
 }
 
 KoUnit KisDocument::unit() const
@@ -1453,8 +1580,15 @@ void KisDocument::slotUndoStackCleanChanged(bool value)
 
 void KisDocument::slotConfigChanged()
 {
-    KisConfig cfg;
-    d->undoStack->setUndoLimit(cfg.undoStackLimit());
+    KisConfig cfg(true);
+
+    if (d->undoStack->undoLimit() != cfg.undoStackLimit()) {
+        if (!d->undoStack->isClean()) {
+            d->undoStack->clear();
+        }
+        d->undoStack->setUndoLimit(cfg.undoStackLimit());
+    }
+
     d->autoSaveDelay = cfg.autoSaveInterval();
     setNormalAutoSaveInterval();
 }
@@ -1474,6 +1608,16 @@ void KisDocument::setGridConfig(const KisGridConfig &config)
     d->gridConfig = config;
 }
 
+QList<KoColorSet *> &KisDocument::paletteList()
+{
+    return d->paletteList;
+}
+
+void KisDocument::setPaletteList(const QList<KoColorSet *> &paletteList)
+{
+    d->paletteList = paletteList;
+}
+
 const KisGuidesConfig& KisDocument::guidesConfig() const
 {
     return d->guidesConfig;
@@ -1487,6 +1631,23 @@ void KisDocument::setGuidesConfig(const KisGuidesConfig &data)
     emit sigGuidesConfigChanged(d->guidesConfig);
 }
 
+const KisMirrorAxisConfig& KisDocument::mirrorAxisConfig() const
+{
+    return d->mirrorAxisConfig;
+}
+
+void KisDocument::setMirrorAxisConfig(const KisMirrorAxisConfig &config)
+{
+    if (d->mirrorAxisConfig == config) {
+        return;
+    }
+
+    d->mirrorAxisConfig = config;
+    setModified(true);
+
+    emit sigMirrorAxisConfigChanged();
+}
+
 void KisDocument::resetURL() {
     setUrl(QUrl());
     setLocalFilePath(QString());
@@ -1549,8 +1710,9 @@ void KisDocument::setLocalFilePath( const QString &localFilePath )
 
 bool KisDocument::openUrlInternal(const QUrl &url)
 {
-    if ( !url.isValid() )
+    if ( !url.isValid() ) {
         return false;
+    }
 
     if (d->m_bAutoDetectedMime) {
         d->mimeType = QByteArray();
@@ -1559,8 +1721,9 @@ bool KisDocument::openUrlInternal(const QUrl &url)
 
     QByteArray mimetype = d->mimeType;
 
-    if ( !closeUrl() )
+    if ( !closeUrl() ) {
         return false;
+    }
 
     d->mimeType = mimetype;
     setUrl(url);
@@ -1595,16 +1758,13 @@ bool KisDocument::openUrlInternal(const QUrl &url)
 bool KisDocument::newImage(const QString& name,
                            qint32 width, qint32 height,
                            const KoColorSpace* cs,
-                           const KoColor &bgColor, bool backgroundAsLayer,
+                           const KoColor &bgColor, KisConfig::BackgroundStyle bgStyle,
                            int numberOfLayers,
                            const QString &description, const double imageResolution)
 {
     Q_ASSERT(cs);
 
-    KisConfig cfg;
-
     KisImageSP image;
-    KisPaintLayerSP layer;
 
     if (!cs) return false;
 
@@ -1621,26 +1781,36 @@ bool KisDocument::newImage(const QString& name,
     documentInfo()->setAboutInfo("title", name);
     documentInfo()->setAboutInfo("abstract", description);
 
-    layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, cs);
-    Q_CHECK_PTR(layer);
+    KisLayerSP layer;
+    if (bgStyle == KisConfig::RASTER_LAYER || bgStyle == KisConfig::FILL_LAYER) {
+        KoColor strippedAlpha = bgColor;
+        strippedAlpha.setOpacity(OPACITY_OPAQUE_U8);
 
-    if (backgroundAsLayer) {
-        image->setDefaultProjectionColor(KoColor(cs));
+        if (bgStyle == KisConfig::RASTER_LAYER) {
+            layer = new KisPaintLayer(image.data(), "Background", OPACITY_OPAQUE_U8, cs);;
+            layer->paintDevice()->setDefaultPixel(strippedAlpha);
+        } else if (bgStyle == KisConfig::FILL_LAYER) {
+            KisFilterConfigurationSP filter_config = KisGeneratorRegistry::instance()->get("color")->defaultConfiguration();
+            filter_config->setProperty("color", strippedAlpha.toQColor());
+            layer = new KisGeneratorLayer(image.data(), "Background Fill", filter_config, image->globalSelection());
+        }
 
-        if (bgColor.opacityU8() == OPACITY_OPAQUE_U8) {
-            layer->paintDevice()->setDefaultPixel(bgColor);
-        } else {
-            // Hack: with a semi-transparent background color, the projection isn't composited right if we just set the default pixel
-            KisFillPainter painter;
-            painter.begin(layer->paintDevice());
-            painter.fillRect(0, 0, width, height, bgColor, bgColor.opacityU8());
+        layer->setOpacity(bgColor.opacityU8());
+
+        if (numberOfLayers > 1) {
+            //Lock bg layer if others are present.
+            layer->setUserLocked(true);
         }
-    } else {
-        image->setDefaultProjectionColor(bgColor);
     }
+    else { // KisConfig::CANVAS_COLOR (needs an unlocked starting layer).
+        image->setDefaultProjectionColor(bgColor);
+        layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, cs);
+    }
+
+    Q_CHECK_PTR(layer);
+    image->addNode(layer.data(), image->rootLayer().data());
     layer->setDirty(QRect(0, 0, width, height));
 
-    image->addNode(layer.data(), image->rootLayer().data());
     setCurrentImage(image);
 
     for(int i = 1; i < numberOfLayers; ++i) {
@@ -1649,6 +1819,7 @@ bool KisDocument::newImage(const QString& name,
         layer->setDirty(QRect(0, 0, width, height));
     }
 
+    KisConfig cfg(false);
     cfg.defImageWidth(width);
     cfg.defImageHeight(height);
     cfg.defImageResolution(imageResolution);
@@ -1656,6 +1827,14 @@ bool KisDocument::newImage(const QString& name,
     cfg.setDefaultColorDepth(image->colorSpace()->colorDepthId().id());
     cfg.defColorProfile(image->colorSpace()->profile()->name());
 
+    KisUsageLogger::log(i18n("Created image \"%1\", %2 * %3 pixels, %4 dpi. Color model: %6 %5 (%7). Layers: %8"
+                             , name
+                             , width, height
+                             , imageResolution * 72.0
+                             , image->colorSpace()->colorModelId().name(), image->colorSpace()->colorDepthId().name()
+                             , image->colorSpace()->profile()->name()
+                             , numberOfLayers));
+
     QApplication::restoreOverrideCursor();
 
     return true;
@@ -1720,7 +1899,7 @@ void KisDocument::setReferenceImagesLayer(KisSharedPtr<KisReferenceImagesLayer>
     d->referenceImagesLayer = layer;
 
     if (d->referenceImagesLayer) {
-        connect(d->referenceImagesLayer, SIGNAL(sigUpdateCanvas(const QRectF&)),
+        connect(d->referenceImagesLayer, SIGNAL(sigUpdateCanvas(QRectF)),
                 this, SIGNAL(sigReferenceImagesChanged()));
     }
 }
