@@ -31,6 +31,8 @@
 #include <QLabel>
 #include <QMouseEvent>
 #include <QDesktopWidget>
+#include <QScreen>
+#include <QWindow>
 
 #include <kis_debug.h>
 
@@ -63,6 +65,7 @@
 #include "kis_selection.h"
 #include "kis_selection_component.h"
 #include "flake/kis_shape_selection.h"
+#include "kis_selection_mask.h"
 #include "kis_image_config.h"
 #include "kis_infinity_manager.h"
 #include "kis_signal_compressor.h"
@@ -96,13 +99,15 @@
 #include "kis_algebra_2d.h"
 #include "kis_image_signal_router.h"
 
+#include "KisSnapPixelStrategy.h"
+
 
 class Q_DECL_HIDDEN KisCanvas2::KisCanvas2Private
 {
 
 public:
 
-    KisCanvas2Private(KoCanvasBase *parent, KisCoordinatesConverter* coordConverter, QPointer<KisView> view, KoCanvasResourceManager* resourceManager)
+    KisCanvas2Private(KoCanvasBase *parent, KisCoordinatesConverter* coordConverter, QPointer<KisView> view, KoCanvasResourceProvider* resourceManager)
         : coordinatesConverter(coordConverter)
         , view(view)
         , shapeManager(parent)
@@ -163,30 +168,30 @@ namespace {
 KoShapeManager* fetchShapeManagerFromNode(KisNodeSP node)
 {
     KoShapeManager *shapeManager = 0;
+    KisSelectionSP selection;
 
-    KisLayer *layer = dynamic_cast<KisLayer*>(node.data());
-
-    if (layer) {
+    if (KisLayer *layer = dynamic_cast<KisLayer*>(node.data())) {
         KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(layer);
         if (shapeLayer) {
             shapeManager = shapeLayer->shapeManager();
 
-        } else {
-            KisSelectionSP selection = layer->selection();
-            if (selection && selection->hasShapeSelection()) {
-                KisShapeSelection *shapeSelection = dynamic_cast<KisShapeSelection*>(selection->shapeSelection());
-                KIS_ASSERT_RECOVER_RETURN_VALUE(shapeSelection, 0);
-
-                shapeManager = shapeSelection->shapeManager();
-            }
         }
+    } else if (KisSelectionMask *mask = dynamic_cast<KisSelectionMask*>(node.data())) {
+        selection = mask->selection();
+    }
+
+    if (!shapeManager && selection && selection->hasShapeSelection()) {
+        KisShapeSelection *shapeSelection = dynamic_cast<KisShapeSelection*>(selection->shapeSelection());
+        KIS_ASSERT_RECOVER_RETURN_VALUE(shapeSelection, 0);
+
+        shapeManager = shapeSelection->shapeManager();
     }
 
     return shapeManager;
 }
 }
 
-KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceManager *resourceManager, KisView *view, KoShapeControllerBase *sc)
+KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisView *view, KoShapeControllerBase *sc)
     : KoCanvasBase(sc, resourceManager)
     , m_d(new KisCanvas2Private(this, coordConverter, view, resourceManager))
 {
@@ -197,20 +202,22 @@ KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResource
      */
     m_d->bootstrapLodBlocked = true;
     connect(view->mainWindow(), SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
+    connect(view->mainWindow(), SIGNAL(screenChanged()), SLOT(slotConfigChanged()));
 
-    KisImageConfig config;
+    KisImageConfig config(false);
 
     m_d->canvasUpdateCompressor.setDelay(1000 / config.fpsLimit());
     m_d->canvasUpdateCompressor.setMode(KisSignalCompressor::FIRST_ACTIVE);
 
     m_d->frameRenderStartCompressor.setDelay(1000 / config.fpsLimit());
     m_d->frameRenderStartCompressor.setMode(KisSignalCompressor::FIRST_ACTIVE);
+    snapGuide()->overrideSnapStrategy(KoSnapGuide::PixelSnapping, new KisSnapPixelStrategy());
 }
 
 void KisCanvas2::setup()
 {
     // a bit of duplication from slotConfigChanged()
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_d->vastScrolling = cfg.vastScrolling();
     m_d->lodAllowedInImage = cfg.levelOfDetailEnabled();
 
@@ -240,9 +247,9 @@ void KisCanvas2::setup()
     connect(kritaShapeController, SIGNAL(selectionChanged()),
             this, SLOT(slotSelectionChanged()));
     connect(kritaShapeController, SIGNAL(selectionContentChanged()),
-            globalShapeManager(), SIGNAL(selectionContentChanged()));
+            selectedShapesProxy(), SIGNAL(selectionContentChanged()));
     connect(kritaShapeController, SIGNAL(currentLayerChanged(const KoShapeLayer*)),
-            globalShapeManager()->selection(), SIGNAL(currentLayerChanged(const KoShapeLayer*)));
+            selectedShapesProxy(), SIGNAL(currentLayerChanged(const KoShapeLayer*)));
 
     connect(&m_d->canvasUpdateCompressor, SIGNAL(timeout()), SLOT(slotDoCanvasUpdate()));
 
@@ -260,7 +267,7 @@ void KisCanvas2::setup()
 
 void KisCanvas2::initializeFpsDecoration()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
 
     const bool shouldShowDebugOverlay =
         (canvasIsOpenGL() && cfg.enableOpenGLFramerateLogging()) ||
@@ -397,12 +404,7 @@ void KisCanvas2::KisCanvas2Private::setActiveShapeManager(KoShapeManager *shapeM
 
 KoShapeManager* KisCanvas2::shapeManager() const
 {
-    KisNodeSP node = m_d->view->currentNode();
-    KoShapeManager *localShapeManager = fetchShapeManagerFromNode(node);
-
-    if (localShapeManager != m_d->currentlyActiveShapeManager) {
-        m_d->setActiveShapeManager(localShapeManager);
-    }
+    KoShapeManager *localShapeManager = this->localShapeManager();
 
     // sanity check for consistency of the local shape manager
     KIS_SAFE_ASSERT_RECOVER (localShapeManager == m_d->currentlyActiveShapeManager) {
@@ -422,6 +424,18 @@ KoShapeManager* KisCanvas2::globalShapeManager() const
     return &m_d->shapeManager;
 }
 
+KoShapeManager *KisCanvas2::localShapeManager() const
+{
+    KisNodeSP node = m_d->view->currentNode();
+    KoShapeManager *localShapeManager = fetchShapeManagerFromNode(node);
+
+    if (localShapeManager != m_d->currentlyActiveShapeManager) {
+        m_d->setActiveShapeManager(localShapeManager);
+    }
+
+    return localShapeManager;
+}
+
 void KisCanvas2::updateInputMethodInfo()
 {
     // TODO call (the protected) QWidget::updateMicroFocus() on the proper canvas widget...
@@ -504,7 +518,7 @@ void KisCanvas2::createQPainterCanvas()
 
 void KisCanvas2::createOpenGLCanvas()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_d->openGLFilterMode = cfg.openGLFilteringMode();
     m_d->currentCanvasIsOpenGL = true;
 
@@ -520,25 +534,28 @@ void KisCanvas2::createCanvas(bool useOpenGL)
     m_d->prescaledProjection = 0;
     m_d->frameCache = 0;
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     QDesktopWidget dw;
     const KoColorProfile *profile = cfg.displayProfile(dw.screenNumber(imageView()));
+    m_d->displayColorConverter.notifyOpenGLCanvasIsActive(useOpenGL && KisOpenGL::hasOpenGL());
     m_d->displayColorConverter.setMonitorProfile(profile);
 
+    if (useOpenGL && !KisOpenGL::hasOpenGL()) {
+        warnKrita << "Tried to create OpenGL widget when system doesn't have OpenGL\n";
+        useOpenGL = false;
+    }
+
+    m_d->displayColorConverter.notifyOpenGLCanvasIsActive(useOpenGL);
+
     if (useOpenGL) {
-        if (KisOpenGL::hasOpenGL()) {
-            createOpenGLCanvas();
-            if (cfg.canvasState() == "OPENGL_FAILED") {
-                // Creating the opengl canvas failed, fall back
-                warnKrita << "OpenGL Canvas initialization returned OPENGL_FAILED. Falling back to QPainter.";
-                createQPainterCanvas();
-            }
-        } else {
-            warnKrita << "Tried to create OpenGL widget when system doesn't have OpenGL\n";
+        createOpenGLCanvas();
+        if (cfg.canvasState() == "OPENGL_FAILED") {
+            // Creating the opengl canvas failed, fall back
+            warnKrita << "OpenGL Canvas initialization returned OPENGL_FAILED. Falling back to QPainter.";
+            m_d->displayColorConverter.notifyOpenGLCanvasIsActive(false);
             createQPainterCanvas();
         }
-    }
-    else {
+    } else {
         createQPainterCanvas();
     }
 
@@ -552,6 +569,7 @@ void KisCanvas2::initializeImage()
 {
     KisImageSP image = m_d->view->image();
 
+    m_d->displayColorConverter.setImageColorSpace(image->colorSpace());
     m_d->coordinatesConverter->setImage(image);
     m_d->toolProxy.initializeImage(image);
 
@@ -561,9 +579,12 @@ void KisCanvas2::initializeImage()
     connect(image->signalRouter(), SIGNAL(sigRequestLodPlanesSyncBlocked(bool)), SLOT(slotSetLodUpdatesBlocked(bool)), Qt::DirectConnection);
 
     connect(image, SIGNAL(sigProofingConfigChanged()), SLOT(slotChangeProofingConfig()));
-    connect(image, SIGNAL(sigSizeChanged(const QPointF&, const QPointF&)), SLOT(startResizingImage()), Qt::DirectConnection);
+    connect(image, SIGNAL(sigSizeChanged(QPointF,QPointF)), SLOT(startResizingImage()), Qt::DirectConnection);
     connect(image->undoAdapter(), SIGNAL(selectionChanged()), SLOT(slotTrySwitchShapeManager()));
 
+    connect(image, SIGNAL(sigColorSpaceChanged(const KoColorSpace*)), SLOT(slotImageColorSpaceChanged()));
+    connect(image, SIGNAL(sigProfileChanged(const KoColorProfile*)), SLOT(slotImageColorSpaceChanged()));
+
     connectCurrentCanvas();
 }
 
@@ -589,7 +610,7 @@ void KisCanvas2::resetCanvas(bool useOpenGL)
     if (!m_d->canvasWidget) {
         return;
     }
-    KisConfig cfg;
+    KisConfig cfg(true);
     bool needReset = (m_d->currentCanvasIsOpenGL != useOpenGL) ||
         (m_d->currentCanvasIsOpenGL &&
          m_d->openGLFilterMode != cfg.openGLFilteringMode());
@@ -604,10 +625,15 @@ void KisCanvas2::resetCanvas(bool useOpenGL)
 
 void KisCanvas2::startUpdateInPatches(const QRect &imageRect)
 {
+    /**
+     * We don't do patched loading for openGL canvas, becasue it loads
+     * the tiles, which are bascially "patches". Therefore, big chunks
+     * of memory are never allocated.
+     */
     if (m_d->currentCanvasIsOpenGL) {
         startUpdateCanvasProjection(imageRect);
     } else {
-        KisImageConfig imageConfig;
+        KisImageConfig imageConfig(true);
         int patchWidth = imageConfig.updatePatchWidth();
         int patchHeight = imageConfig.updatePatchHeight();
 
@@ -637,6 +663,19 @@ QSharedPointer<KisDisplayFilter> KisCanvas2::displayFilter() const
     return m_d->displayColorConverter.displayFilter();
 }
 
+void KisCanvas2::slotImageColorSpaceChanged()
+{
+    KisImageSP image = this->image();
+
+    m_d->view->viewManager()->blockUntilOperationsFinishedForced(image);
+
+    m_d->displayColorConverter.setImageColorSpace(image->colorSpace());
+
+    image->barrierLock();
+    m_d->canvasWidget->notifyImageColorSpaceChanged(image->colorSpace());
+    image->unlock();
+}
+
 KisDisplayColorConverter* KisCanvas2::displayColorConverter() const
 {
     return &m_d->displayColorConverter;
@@ -655,8 +694,7 @@ void KisCanvas2::setProofingOptions(bool softProof, bool gamutCheck)
 {
     m_d->proofingConfig = this->image()->proofingConfiguration();
     if (!m_d->proofingConfig) {
-        qDebug()<<"Canvas: No proofing config found, generating one.";
-        KisImageConfig cfg;
+        KisImageConfig cfg(false);
         m_d->proofingConfig = cfg.defaultProofingconfiguration();
     }
     KoColorConversionTransformation::ConversionFlags conversionFlags = m_d->proofingConfig->conversionFlags;
@@ -719,7 +757,7 @@ KisProofingConfigurationSP KisCanvas2::proofingConfiguration() const
     if (!m_d->proofingConfig) {
         m_d->proofingConfig = this->image()->proofingConfiguration();
         if (!m_d->proofingConfig) {
-            m_d->proofingConfig = KisImageConfig().defaultProofingconfiguration();
+            m_d->proofingConfig = KisImageConfig(true).defaultProofingconfiguration();
         }
     }
     return m_d->proofingConfig;
@@ -780,7 +818,15 @@ void KisCanvas2::updateCanvasProjection()
     bool shouldExplicitlyIssueUpdates = false;
 
     QVector<KisUpdateInfoSP> infoObjects;
-    while (KisUpdateInfoSP info = m_d->projectionUpdatesCompressor.takeUpdateInfo()) {
+    KisUpdateInfoList originalInfoObjects;
+    m_d->projectionUpdatesCompressor.takeUpdateInfo(originalInfoObjects);
+
+    for (auto it = originalInfoObjects.constBegin();
+         it != originalInfoObjects.constEnd();
+         ++it) {
+
+        KisUpdateInfoSP info = *it;
+
         const KisMarkerUpdateInfo *batchInfo = dynamic_cast<const KisMarkerUpdateInfo*>(info.data());
         if (batchInfo) {
             if (!infoObjects.isEmpty()) {
@@ -968,7 +1014,7 @@ void KisCanvas2::notifyLevelOfDetailChange()
 
     const qreal effectiveZoom = m_d->coordinatesConverter->effectiveZoom();
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     const int maxLod = cfg.numMipmapLevels();
 
     const int lod = KisLodTransform::scaleToLod(effectiveZoom, maxLod);
@@ -1011,7 +1057,16 @@ KisImageWSP KisCanvas2::currentImage() const
 void KisCanvas2::documentOffsetMoved(const QPoint &documentOffset)
 {
     QPointF offsetBefore = m_d->coordinatesConverter->imageRectInViewportPixels().topLeft();
-    m_d->coordinatesConverter->setDocumentOffset(documentOffset);
+
+    // The given offset is in widget logical pixels. In order to prevent fuzzy
+    // canvas rendering at 100% pixel-perfect zoom level when devicePixelRatio
+    // is not integral, we adjusts the offset to map to whole device pixels.
+    //
+    // FIXME: This is a temporary hack for fixing the canvas under fractional
+    //        DPI scaling before a new coordinate system is introduced.
+    QPointF offsetAdjusted = m_d->coordinatesConverter->snapToDevicePixel(documentOffset);
+
+    m_d->coordinatesConverter->setDocumentOffset(offsetAdjusted);
     QPointF offsetAfter = m_d->coordinatesConverter->imageRectInViewportPixels().topLeft();
 
     QPointF moveOffset = offsetAfter - offsetBefore;
@@ -1028,11 +1083,28 @@ void KisCanvas2::documentOffsetMoved(const QPoint &documentOffset)
 
 void KisCanvas2::slotConfigChanged()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_d->vastScrolling = cfg.vastScrolling();
 
     resetCanvas(cfg.useOpenGL());
-    slotSetDisplayProfile(cfg.displayProfile(QApplication::desktop()->screenNumber(this->canvasWidget())));
+
+    // HACK: Sometimes screenNumber(this->canvasWidget()) is not able to get the
+    //       proper screenNumber when moving the window across screens. Using
+    //       the coordinates should be able to work around this.
+    // FIXME: We should change to associate the display profiles with the screen
+    //        model and serial number instead. See https://bugs.kde.org/show_bug.cgi?id=407498
+    QScreen *canvasScreen = this->canvasWidget()->window()->windowHandle()->screen();
+    QPoint canvasScreenCenter = canvasScreen->geometry().center();
+    int canvasScreenNumber = QApplication::desktop()->screenNumber(canvasScreenCenter);
+    if (canvasScreenNumber == -1) {
+        // Fall back to the old way of getting the screenNumber
+        canvasScreenNumber = QApplication::desktop()->screenNumber(this->canvasWidget());
+    }
+    if (canvasScreenNumber != -1) {
+        setDisplayProfile(cfg.displayProfile(canvasScreenNumber));
+    } else {
+        warnUI << "Failed to get screenNumber for updating display profile.";
+    }
 
     initializeFpsDecoration();
 }
@@ -1044,7 +1116,7 @@ void KisCanvas2::refetchDataFromImage()
     startUpdateInPatches(image->bounds());
 }
 
-void KisCanvas2::slotSetDisplayProfile(const KoColorProfile *monitorProfile)
+void KisCanvas2::setDisplayProfile(const KoColorProfile *monitorProfile)
 {
     if (m_d->displayColorConverter.monitorProfile() == monitorProfile) return;
 
@@ -1053,7 +1125,7 @@ void KisCanvas2::slotSetDisplayProfile(const KoColorProfile *monitorProfile)
     {
         KisImageSP image = this->image();
         KisImageBarrierLocker l(image);
-        m_d->canvasWidget->setDisplayProfile(&m_d->displayColorConverter);
+        m_d->canvasWidget->setDisplayColorConverter(&m_d->displayColorConverter);
     }
 
     refetchDataFromImage();
@@ -1190,7 +1262,7 @@ void KisCanvas2::setLodAllowedInCanvas(bool value)
 
     notifyLevelOfDetailChange();
 
-    KisConfig cfg;
+    KisConfig cfg(false);
     cfg.setLevelOfDetailEnabled(m_d->lodAllowedInImage);
 }
 
