@@ -21,6 +21,7 @@
 
 #include "kis_coordinates_converter.h"
 
+#include <QtMath>
 #include <QTransform>
 #include <KoViewConverter.h>
 
@@ -68,7 +69,7 @@ struct KisCoordinatesConverter::Private {
 
 QPointF KisCoordinatesConverter::centeringCorrection() const
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
 
     QSize documentSize = imageRectInWidgetPixels().toAlignedRect().size();
     QPointF dPoint(documentSize.width(), documentSize.height());
@@ -105,8 +106,8 @@ QPointF KisCoordinatesConverter::centeringCorrection() const
 
 void KisCoordinatesConverter::correctOffsetToTransformation()
 {
-    m_d->documentOffset = -(imageRectInWidgetPixels().topLeft() -
-          centeringCorrection()).toPoint();
+    m_d->documentOffset = snapToDevicePixel(-(imageRectInWidgetPixels().topLeft() -
+          centeringCorrection()));
 }
 
 void KisCoordinatesConverter::correctTransformationToOffset()
@@ -150,7 +151,7 @@ KisCoordinatesConverter::~KisCoordinatesConverter()
     delete m_d;
 }
 
-void KisCoordinatesConverter::setCanvasWidgetSize(QSize size)
+void KisCoordinatesConverter::setCanvasWidgetSize(QSizeF size)
 {
     m_d->canvasWidgetSize = size;
     recalculateTransformations();
@@ -167,7 +168,7 @@ void KisCoordinatesConverter::setImage(KisImageWSP image)
     recalculateTransformations();
 }
 
-void KisCoordinatesConverter::setDocumentOffset(const QPoint& offset)
+void KisCoordinatesConverter::setDocumentOffset(const QPointF& offset)
 {
     QPointF diff = m_d->documentOffset - offset;
 
@@ -176,6 +177,11 @@ void KisCoordinatesConverter::setDocumentOffset(const QPoint& offset)
     recalculateTransformations();
 }
 
+qreal KisCoordinatesConverter::devicePixelRatio() const
+{
+    return m_d->devicePixelRatio;
+}
+
 QPoint KisCoordinatesConverter::documentOffset() const
 {
     return QPoint(int(m_d->documentOffset.x()), int(m_d->documentOffset.y()));
@@ -462,3 +468,19 @@ void KisCoordinatesConverter::imagePhysicalScale(qreal *scaleX, qreal *scaleY) c
     *scaleX *= m_d->devicePixelRatio;
     *scaleY *= m_d->devicePixelRatio;
 }
+
+/**
+ * @brief Adjust a given pair of coordinates to the nearest device pixel
+ *        according to the value of `devicePixelRatio`.
+ * @param point a point in logical pixel space
+ * @return The point in logical pixel space but adjusted to the nearest device
+ *         pixel
+ */
+
+QPointF KisCoordinatesConverter::snapToDevicePixel(const QPointF &point) const
+{
+    QPoint devicePixel = (point * m_d->devicePixelRatio).toPoint();
+    // These adjusted coords will be in logical pixel but is aligned in device
+    // pixel space for pixel-perfect rendering.
+    return QPointF(devicePixel) / m_d->devicePixelRatio;
+}
