@@ -31,6 +31,8 @@
 
 #include <QDebug>
 
+#include "kis_debug.h"
+
 class LcmsColorProfileContainer::Private
 {
 public:
@@ -67,6 +69,8 @@ public:
     bool isAbsoluteCLUT;
     bool isSaturationCLUT;
     bool isMatrixShaper;
+
+    QByteArray uniqueId;
 };
 
 LcmsColorProfileContainer::LcmsColorProfileContainer()
@@ -122,6 +126,7 @@ bool LcmsColorProfileContainer::init()
 
     d->profile = cmsOpenProfileFromMem((void *)d->data->rawData().constData(), d->data->rawData().size());
 
+
 #ifndef NDEBUG
     if (d->data->rawData().size() == 4096) {
         qWarning() << "Profile has a size of 4096, which is suspicious and indicates a possible misuse of QIODevice::read(int), check your code.";
@@ -135,7 +140,7 @@ bool LcmsColorProfileContainer::init()
         cmsGetProfileInfo(d->profile, cmsInfoDescription, cmsNoLanguage, cmsNoCountry, buffer, _BUFFER_SIZE_);
         d->name = QString::fromWCharArray(buffer);
 
-        //apparantly this should give us a localised string??? Not sure about this.
+        //apparently this should give us a localised string??? Not sure about this.
         cmsGetProfileInfo(d->profile, cmsInfoModel, cmsNoLanguage, cmsNoCountry, buffer, _BUFFER_SIZE_);
         d->productDescription = QString::fromWCharArray(buffer);
 
@@ -206,15 +211,15 @@ bool LcmsColorProfileContainer::init()
             d->redTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigRedTRCTag));
             d->greenTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigGreenTRCTag));
             d->blueTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigBlueTRCTag));
-            d->redTRCReverse = cmsReverseToneCurve(d->redTRC);
-            d->greenTRCReverse = cmsReverseToneCurve(d->greenTRC);
-            d->blueTRCReverse = cmsReverseToneCurve(d->blueTRC);
-            d->hasTRC = true;
+            if (d->redTRC) d->redTRCReverse = cmsReverseToneCurve(d->redTRC);
+            if (d->greenTRC) d->greenTRCReverse = cmsReverseToneCurve(d->greenTRC);
+            if (d->blueTRC) d->blueTRCReverse = cmsReverseToneCurve(d->blueTRC);
+            d->hasTRC = (d->redTRC && d->greenTRC && d->blueTRC && d->redTRCReverse && d->greenTRCReverse && d->blueTRCReverse);
 
         } else if (cmsIsTag(d->profile, cmsSigGrayTRCTag)) {
             d->grayTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigGrayTRCTag));
-            d->grayTRCReverse = cmsReverseToneCurve(d->grayTRC);
-            d->hasTRC = true;
+            if (d->grayTRC) d->grayTRCReverse = cmsReverseToneCurve(d->grayTRC);
+            d->hasTRC = (d->grayTRC && d->grayTRCReverse);
         } else {
             d->hasTRC = false;
         }
@@ -529,3 +534,27 @@ QString LcmsColorProfileContainer::info() const
 {
     return d->productDescription;
 }
+
+QByteArray LcmsColorProfileContainer::getProfileUniqueId() const
+{
+    if (d->uniqueId.isEmpty() && d->profile) {
+        QByteArray id(sizeof(cmsProfileID), 0);
+        cmsGetHeaderProfileID(d->profile, (quint8*)id.data());
+
+        bool isNull = std::all_of(id.constBegin(),
+                                  id.constEnd(),
+                                  [](char c) {return c == 0;});
+        if (isNull) {
+            if (cmsMD5computeID(d->profile)) {
+                cmsGetHeaderProfileID(d->profile, (quint8*)id.data());
+                isNull = false;
+            }
+        }
+
+        if (!isNull) {
+            d->uniqueId = id;
+        }
+    }
+
+    return d->uniqueId;
+}
