@@ -22,7 +22,6 @@
 #include <QQmlContext>
 #include <QAction>
 #include <QUrl>
-#include <QAction>
 #include <QKeyEvent>
 #include <QApplication>
 
@@ -45,7 +44,6 @@
 #include <KisMimeDatabase.h>
 #include <kis_action_manager.h>
 #include <kis_action.h>
-#include <kis_config.h>
 
 #include <Theme.h>
 #include <Settings.h>
@@ -103,7 +101,7 @@ TouchDockerDock::TouchDockerDock()
                                                << "previous_preset"
                                                << "clear";
 
-    QStringList mapping = KisConfig().readEntry<QString>("touchdockermapping", defaultMapping.join(',')).split(',');
+    QStringList mapping = KisConfig(true).readEntry<QString>("touchdockermapping", defaultMapping.join(',')).split(',');
     for (int i = 0; i < 8; ++i) {
         if (i < mapping.size()) {
             d->buttonMapping[QString("button%1").arg(i + 1)] = mapping[i];
@@ -315,10 +313,37 @@ void TouchDockerDock::showFileOpenDialog()
 
 void TouchDockerDock::showFileSaveAsDialog()
 {
-    if (!d->openDialog) {
-        d->openDialog = createDialog("qrc:/saveasdialog.qml");
+    if (!d->saveAsDialog) {
+        d->saveAsDialog = createDialog("qrc:/saveasdialog.qml");
     }
-    d->openDialog->exec();
+    d->saveAsDialog->exec();
+}
+
+void TouchDockerDock::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::PaletteChange) {
+        m_quickWidget->setSource(QUrl("qrc:/touchstrip.qml"));
+        event->accept();
+    } else {
+        event->ignore();
+    }
+}
+
+void TouchDockerDock::tabletEvent(QTabletEvent *event)
+{
+#ifdef Q_OS_WIN
+    /**
+     * On Windows (only in WinInk mode), unless we accept the tablet event,
+     * OS will start windows gestures, like click+hold for right click.
+     * It will block any mouse events generation.
+     *
+     * In our own (hacky) implementation, if we accept the event, we block
+     * the gesture, but still generate a fake mouse event.
+     */
+    event->accept();
+#else
+    QDockWidget::tabletEvent(event);
+#endif
 }
 
 KoDialog *TouchDockerDock::createDialog(const QString qml)
