@@ -37,6 +37,8 @@ public:
     RectangleShape();
     ~RectangleShape() override;
 
+    KoShape* cloneShape() const override;
+
     /// Returns the corner radius in x-direction
     qreal cornerRadiusX() const;
 
@@ -79,6 +81,7 @@ public:
     bool loadSvg(const KoXmlElement &element, SvgLoadingContext &context) override;
 
 protected:
+    RectangleShape(const RectangleShape &rhs);
 
     void moveHandleAction(int handleId, const QPointF &point, Qt::KeyboardModifiers modifiers = Qt::NoModifier) override;
     void updatePath(const QSizeF &size) override;
