@@ -29,9 +29,10 @@ class KisCSVExport : public KisImportExportFilter
     Q_OBJECT
 public:
     KisCSVExport(QObject *parent, const QVariantList &);
-    virtual ~KisCSVExport();
+    ~KisCSVExport() override;
 public:
-    virtual KisImportExportFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
+    KisImportExportFilter::ConversionStatus convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration = 0) override;
+    void initializeCapabilities() override;
 };
 
 #endif
