@@ -34,10 +34,11 @@
 #include <KoFileDialog.h>
 #include <KoStore.h>
 #include <KoColorSpace.h>
+#include <KoShapeControllerBase.h>
 
 // kritaimage
-#include <metadata/kis_meta_data_io_backend.h>
-#include <metadata/kis_meta_data_store.h>
+#include <kis_meta_data_io_backend.h>
+#include <kis_meta_data_store.h>
 #include <kis_types.h>
 #include <kis_node_visitor.h>
 #include <kis_image.h>
@@ -64,11 +65,14 @@
 #include "kis_dom_utils.h"
 #include "kis_raster_keyframe_channel.h"
 #include "kis_paint_device_frames_interface.h"
+#include "kis_filter_registry.h"
+
 
 using namespace KRA;
 
 QString expandEncodedDirectory(const QString& _intern)
 {
+
     QString intern = _intern;
 
     QString result;
@@ -83,28 +87,30 @@ QString expandEncodedDirectory(const QString& _intern)
     if (!intern.isEmpty() && QChar(intern.at(0)).isDigit())
         result += "part";
     result += intern;
+
+
     return result;
 }
 
 
 KisKraLoadVisitor::KisKraLoadVisitor(KisImageSP image,
                                      KoStore *store,
+                                     KoShapeControllerBase *shapeController,
                                      QMap<KisNode *, QString> &layerFilenames,
                                      QMap<KisNode *, QString> &keyframeFilenames,
                                      const QString & name,
-                                     int syntaxVersion) :
-        KisNodeVisitor(),
-        m_layerFilenames(layerFilenames),
-        m_keyframeFilenames(keyframeFilenames)
+                                     int syntaxVersion)
+    : KisNodeVisitor()
+    , m_image(image)
+    , m_store(store)
+    , m_external(false)
+    , m_layerFilenames(layerFilenames)
+    , m_keyframeFilenames(keyframeFilenames)
+    , m_name(name)
+    , m_shapeController(shapeController)
 {
-    m_external = false;
-    m_image = image;
-    m_store = store;
-    m_name = name;
     m_store->pushDirectory();
-    if (m_name.startsWith("/")) {
-        m_name.remove(0, 1);
-    }
+
     if (!m_store->enterDirectory(m_name)) {
         QStringList directories = m_store->directoryList();
         dbgKrita << directories;
@@ -187,7 +193,6 @@ bool KisKraLoadVisitor::visit(KisPaintLayer *layer)
 {
     loadNodeKeyframes(layer);
 
-    dbgFile << "Visit: " << layer->name() << " colorSpace: " << layer->colorSpace()->id();
     if (!loadPaintDevice(layer->paintDevice(), getLocation(layer))) {
         return false;
     }
@@ -259,7 +264,8 @@ bool KisKraLoadVisitor::visit(KisAdjustmentLayer* layer)
         return false;
     }
 
-    loadFilterConfiguration(layer->filter().data(), getLocation(layer, DOT_FILTERCONFIG));
+    loadFilterConfiguration(layer->filter(), getLocation(layer, DOT_FILTERCONFIG));
+    fixOldFilterConfigurations(layer->filter());
 
     result = visitAll(layer);
     return result;
@@ -274,8 +280,13 @@ bool KisKraLoadVisitor::visit(KisGeneratorLayer *layer)
 
     loadNodeKeyframes(layer);
     result = loadSelection(getLocation(layer), layer->internalSelection());
-    result = loadFilterConfiguration(layer->filter().data(), getLocation(layer, DOT_FILTERCONFIG));
-    layer->update();
+
+    // HACK ALERT: we set the same filter again to ensure the layer
+    // is correctly updated
+    KisFilterConfigurationSP filter = layer->filter();
+    result = loadFilterConfiguration(filter.data(), getLocation(layer, DOT_FILTERCONFIG));
+    layer->setFilter(filter);
+
     result = visitAll(layer);
     return result;
 }
@@ -327,7 +338,8 @@ bool KisKraLoadVisitor::visit(KisFilterMask *mask)
 
     bool result = true;
     result = loadSelection(getLocation(mask), mask->selection());
-    result = loadFilterConfiguration(mask->filter().data(), getLocation(mask, DOT_FILTERCONFIG));
+    result = loadFilterConfiguration(mask->filter(), getLocation(mask, DOT_FILTERCONFIG));
+    fixOldFilterConfigurations(mask->filter());
     return result;
 }
 
@@ -487,7 +499,7 @@ bool KisKraLoadVisitor::loadPaintDevice(KisPaintDeviceSP device, const QString&
         for (int i = 0; i < frames.count(); i++) {
             int id = frames[i];
             if (keyframeChannel->frameFilename(id).isEmpty()) {
-                m_warningMessages << i18n("Could not find keyframe pixel data for frame %1 in %2.").arg(id).arg(location);
+                m_warningMessages << i18n("Could not find keyframe pixel data for frame %1 in %2.", id, location);
             }
             else {
                 Q_ASSERT(!keyframeChannel->frameFilename(id).isEmpty());
@@ -495,7 +507,7 @@ bool KisKraLoadVisitor::loadPaintDevice(KisPaintDeviceSP device, const QString&
                 Q_ASSERT(!frameFilename.isEmpty());
 
                 if (!loadPaintDeviceFrame(device, frameFilename, FramedDevicePolicy(id))) {
-                    m_warningMessages << i18n("Could not load keyframe pixel data for frame %1 in %2.").arg(id).arg(location);
+                    m_warningMessages << i18n("Could not load keyframe pixel data for frame %1 in %2.", id, location);
                 }
             }
         }
@@ -583,6 +595,18 @@ bool KisKraLoadVisitor::loadFilterConfiguration(KisFilterConfigurationSP kfc, co
     return true;
 }
 
+void KisKraLoadVisitor::fixOldFilterConfigurations(KisFilterConfigurationSP kfc)
+{
+    KisFilterSP filter = KisFilterRegistry::instance()->value(kfc->name());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(filter);
+
+    if (!filter->configurationAllowedForMask(kfc)) {
+        filter->fixLoadedFilterConfigurationForMasks(kfc);
+    }
+
+    KIS_SAFE_ASSERT_RECOVER_NOOP(filter->configurationAllowedForMask(kfc));
+}
+
 bool KisKraLoadVisitor::loadMetaData(KisNode* node)
 {
     KisLayer* layer = qobject_cast<KisLayer*>(node);
@@ -643,7 +667,7 @@ bool KisKraLoadVisitor::loadSelection(const QString& location, KisSelectionSP ds
         m_store->pushDirectory();
         m_store->enterDirectory(shapeSelectionLocation) ;
 
-        KisShapeSelection* shapeSelection = new KisShapeSelection(m_image, dstSelection);
+        KisShapeSelection* shapeSelection = new KisShapeSelection(m_shapeController, m_image, dstSelection);
         dstSelection->setShapeSelection(shapeSelection);
         result = shapeSelection->loadSelection(m_store);
         m_store->popDirectory();
