@@ -40,9 +40,9 @@
 #include <kis_paint_layer.h>
 #include <kis_group_layer.h>
 #include <kis_config.h>
-#include <metadata/kis_meta_data_store.h>
-#include <metadata/kis_meta_data_filter_registry_model.h>
-#include <metadata/kis_exif_info_visitor.h>
+#include <kis_meta_data_store.h>
+#include <kis_meta_data_filter_registry_model.h>
+#include <kis_exif_info_visitor.h>
 #include "kis_png_converter.h"
 #include <kis_iterator_ng.h>
 
@@ -73,6 +73,7 @@ KisImportExportFilter::ConversionStatus KisPNGExport::convert(KisDocument *docum
     options.forceSRGB = configuration->getBool("forceSRGB", true);
     options.storeAuthor = configuration->getBool("storeAuthor", false);
     options.storeMetaData = configuration->getBool("storeMetaData", false);
+    options.saveAsHDR = configuration->getBool("saveAsHDR", false);
 
     vKisAnnotationSP_it beginIt = image->beginAnnotations();
     vKisAnnotationSP_it endIt = image->endAnnotations();
@@ -119,6 +120,7 @@ KisPropertiesConfigurationSP KisPNGExport::defaultConfiguration(const QByteArray
     cfg->setProperty("transparencyFillcolor", v);
     cfg->setProperty("saveSRGBProfile", false);
     cfg->setProperty("forceSRGB", true);
+    cfg->setProperty("saveAsHDR", false);
     cfg->setProperty("storeMetaData", false);
     cfg->setProperty("storeAuthor", false);
 
@@ -142,6 +144,14 @@ void KisPNGExport::initializeCapabilities()
     addSupportedColorModels(supportedColorModels, "PNG");
 }
 
+KisWdgOptionsPNG::KisWdgOptionsPNG(QWidget *parent)
+    : KisConfigWidget(parent)
+{
+    setupUi(this);
+
+    connect(chkSaveAsHDR, SIGNAL(toggled(bool)), this, SLOT(slotUseHDRChanged(bool)));
+}
+
 void KisWdgOptionsPNG::setConfiguration(const KisPropertiesConfigurationSP cfg)
 {
     // the export manager should have prepared some info for us!
@@ -181,6 +191,9 @@ void KisWdgOptionsPNG::setConfiguration(const KisPropertiesConfigurationSP cfg)
     //chkForceSRGB->setEnabled(!sRGB);
     chkForceSRGB->setChecked(cfg->getBool("forceSRGB", false));
 
+    chkSaveAsHDR->setChecked(cfg->getBool("saveAsHDR", false));
+    slotUseHDRChanged(chkSaveAsHDR->isChecked());
+
     chkAuthor->setChecked(cfg->getBool("storeAuthor", false));
     chkMetaData->setChecked(cfg->getBool("storeMetaData", false));
 
@@ -198,12 +211,14 @@ KisPropertiesConfigurationSP KisWdgOptionsPNG::configuration() const
     bool alpha = this->alpha->isChecked();
     bool interlace = interlacing->isChecked();
     int compression = (int)compressionLevel->value();
-    bool tryToSaveAsIndexed = this->tryToSaveAsIndexed->isChecked();
-    bool saveSRGB = chkSRGB->isChecked();
-    bool forceSRGB = chkForceSRGB->isChecked();
+    bool saveAsHDR = chkSaveAsHDR->isChecked();
+    bool tryToSaveAsIndexed = !saveAsHDR && this->tryToSaveAsIndexed->isChecked();
+    bool saveSRGB = !saveAsHDR && chkSRGB->isChecked();
+    bool forceSRGB = !saveAsHDR && chkForceSRGB->isChecked();
     bool storeAuthor = chkAuthor->isChecked();
     bool storeMetaData = chkMetaData->isChecked();
 
+
     QVariant transparencyFillcolor;
     transparencyFillcolor.setValue(bnTransparencyFillColor->color());
 
@@ -212,6 +227,7 @@ KisPropertiesConfigurationSP KisWdgOptionsPNG::configuration() const
     cfg->setProperty("compression", compression);
     cfg->setProperty("interlaced", interlace);
     cfg->setProperty("transparencyFillcolor", transparencyFillcolor);
+    cfg->setProperty("saveAsHDR", saveAsHDR);
     cfg->setProperty("saveSRGBProfile", saveSRGB);
     cfg->setProperty("forceSRGB", forceSRGB);
     cfg->setProperty("storeAuthor", storeAuthor);
@@ -225,5 +241,12 @@ void KisWdgOptionsPNG::on_alpha_toggled(bool checked)
     bnTransparencyFillColor->setEnabled(!checked);
 }
 
+void KisWdgOptionsPNG::slotUseHDRChanged(bool value)
+{
+    tryToSaveAsIndexed->setDisabled(value);
+    chkForceSRGB->setDisabled(value);
+    chkSRGB->setDisabled(value);
+}
+
 #include "kis_png_export.moc"
 
