@@ -29,7 +29,11 @@
 #include <QPointer>
 
 class KoSelection;
-class KoCanvasResourceManager;
+class KoCanvasResourceProvider;
+
+static const struct DecoratorIconPositions {
+    QPoint uiOffset = QPoint(0, 40);
+} decoratorIconPositions;
 
 /**
  * The SelectionDecorator is used to paint extra user-interface items on top of a selection.
@@ -43,7 +47,7 @@ public:
      * @param rotationHandles if true; the rotation handles will be drawn
      * @param shearHandles if true; the shearhandles will be drawn
      */
-    SelectionDecorator(KoCanvasResourceManager *resourceManager);
+    SelectionDecorator(KoCanvasResourceProvider *resourceManager);
     ~SelectionDecorator() {}
 
     /**
@@ -77,6 +81,8 @@ public:
      */
     void setShowStrokeFillGradientHandles(bool value);
 
+    void setForceShapeOutlines(bool value);
+
 private:
     void paintGradientHandles(KoShape *shape, KoFlake::FillVariant fillVariant, QPainter &painter, const KoViewConverter &converter);
 
@@ -87,6 +93,7 @@ private:
     int m_lineWidth;
     bool m_showFillGradientHandles;
     bool m_showStrokeFillGradientHandles;
+    bool m_forceShapeOutlines;
 };
 
 #endif
