@@ -91,7 +91,6 @@
 KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     : KisTool(canvas, KisCursor::rotateCursor())
     , m_workRecursively(true)
-    , m_changesTracker(&m_transaction)
     , m_warpStrategy(
         new KisWarpTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
@@ -142,7 +141,7 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     connect(m_warpStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_cageStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_liquifyStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
-    connect(m_liquifyStrategy.data(), SIGNAL(requestCursorOutlineUpdate(const QPointF&)), SLOT(cursorOutlineUpdateRequested(const QPointF&)));
+    connect(m_liquifyStrategy.data(), SIGNAL(requestCursorOutlineUpdate(QPointF)), SLOT(cursorOutlineUpdateRequested(QPointF)));
     connect(m_liquifyStrategy.data(), SIGNAL(requestUpdateOptionWidget()), SLOT(updateOptionWidget()));
     connect(m_freeStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_freeStrategy.data(), SIGNAL(requestResetRotationCenterButtons()), SLOT(resetRotationCenterButtonsRequested()));
@@ -150,8 +149,8 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     connect(m_perspectiveStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_perspectiveStrategy.data(), SIGNAL(requestShowImageTooBig(bool)), SLOT(imageTooBigRequested(bool)));
 
-    connect(&m_changesTracker, SIGNAL(sigConfigChanged()),
-            this, SLOT(slotTrackerChangedConfig()));
+    connect(&m_changesTracker, SIGNAL(sigConfigChanged(KisToolChangesTrackerDataSP)),
+            this, SLOT(slotTrackerChangedConfig(KisToolChangesTrackerDataSP)));
 }
 
 KisToolTransform::~KisToolTransform()
@@ -631,7 +630,7 @@ void KisToolTransform::setWarpPointDensity( int density )
     m_optionsWidget->slotSetWarpDensity(density);
 }
 
-bool KisToolTransform::tryInitTransformModeFromNode(KisNodeSP node)
+bool KisToolTransform::tryInitArgsFromNode(KisNodeSP node)
 {
     bool result = false;
 
@@ -646,7 +645,6 @@ bool KisToolTransform::tryInitTransformModeFromNode(KisNodeSP node)
 
         if (adapter) {
             m_currentArgs = adapter->transformArgs();
-            initGuiAfterTransformMode();
             result = true;
         }
     }
@@ -684,7 +682,7 @@ bool KisToolTransform::tryFetchArgsFromCommandAndUndo(ToolTransformArgs *args, T
     return result;
 }
 
-void KisToolTransform::initTransformMode(ToolTransformArgs::TransformMode mode)
+void KisToolTransform::resetArgsForMode(ToolTransformArgs::TransformMode mode)
 {
     // NOTE: we are requesting an old value of m_currentArgs variable
     //       here, which is global, don't forget about this on higher
@@ -713,7 +711,11 @@ void KisToolTransform::initTransformMode(ToolTransformArgs::TransformMode mode)
     } else if (mode == ToolTransformArgs::PERSPECTIVE_4POINT) {
         m_currentArgs.setMode(ToolTransformArgs::PERSPECTIVE_4POINT);
     }
+}
 
+void KisToolTransform::initTransformMode(ToolTransformArgs::TransformMode mode)
+{
+    resetArgsForMode(mode);
     initGuiAfterTransformMode();
 }
 
@@ -737,7 +739,7 @@ void KisToolTransform::updateSelectionPath()
 
     if (selection && selection->outlineCacheValid()) {
         selectionOutline = selection->outlineCache();
-    } else {
+    } else if (m_selectedPortionCache) {
         selectionOutline.addRect(m_selectedPortionCache->exactBounds());
     }
 
@@ -760,25 +762,27 @@ void KisToolTransform::initThumbnailImage(KisPaintDeviceSP previewDevice)
     int x, y, w, h;
     srcRect.getRect(&x, &y, &w, &h);
 
-    if (w > maxSize || h > maxSize) {
-        qreal scale = qreal(maxSize) / (w > h ? w : h);
-        QTransform scaleTransform = QTransform::fromScale(scale, scale);
+    if (m_selectedPortionCache) {
+        if (w > maxSize || h > maxSize) {
+            qreal scale = qreal(maxSize) / (w > h ? w : h);
+            QTransform scaleTransform = QTransform::fromScale(scale, scale);
 
-        QRect thumbRect = scaleTransform.mapRect(m_transaction.originalRect()).toAlignedRect();
+            QRect thumbRect = scaleTransform.mapRect(m_transaction.originalRect()).toAlignedRect();
 
-        origImg = m_selectedPortionCache->
-            createThumbnail(thumbRect.width(),
-                            thumbRect.height(),
-                            srcRect, 1,
-                            KoColorConversionTransformation::internalRenderingIntent(),
-                            KoColorConversionTransformation::internalConversionFlags());
-        thumbToImageTransform = scaleTransform.inverted();
+            origImg = m_selectedPortionCache->
+                    createThumbnail(thumbRect.width(),
+                                    thumbRect.height(),
+                                    srcRect, 1,
+                                    KoColorConversionTransformation::internalRenderingIntent(),
+                                    KoColorConversionTransformation::internalConversionFlags());
+            thumbToImageTransform = scaleTransform.inverted();
 
-    } else {
-        origImg = m_selectedPortionCache->convertToQImage(0, x, y, w, h,
-                                                            KoColorConversionTransformation::internalRenderingIntent(),
-                                                            KoColorConversionTransformation::internalConversionFlags());
-        thumbToImageTransform = QTransform();
+        } else {
+            origImg = m_selectedPortionCache->convertToQImage(0, x, y, w, h,
+                                                              KoColorConversionTransformation::internalRenderingIntent(),
+                                                              KoColorConversionTransformation::internalConversionFlags());
+            thumbToImageTransform = QTransform();
+        }
     }
 
     // init both strokes since the thumbnail is initialized only once
@@ -812,7 +816,11 @@ void KisToolTransform::requestUndoDuringStroke()
 {
     if (!m_strokeData.strokeId()) return;
 
-    m_changesTracker.requestUndo();
+    if (m_changesTracker.isEmpty()) {
+        cancelStroke();
+    } else {
+        m_changesTracker.requestUndo();
+    }
 }
 
 void KisToolTransform::requestStrokeEnd()
@@ -833,15 +841,6 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
 {
     Q_ASSERT(!m_strokeData.strokeId());
 
-    KisResourcesSnapshotSP resources =
-            new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
-
-    KisNodeSP currentNode = resources->currentNode();
-
-    if (!currentNode || !currentNode->isEditable()) {
-        return;
-    }
-
     /**
      * FIXME: The transform tool is not completely asynchronous, it
      * needs the content of the layer for creation of the stroke
@@ -851,12 +850,45 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
      * stroke and pass it to the tool somehow. But currently, we will
      * just disable starting a new stroke asynchronously
      */
+
     if (image()->tryBarrierLock()) {
         image()->unlock();
     } else {
         return;
     }
 
+
+    // set up and null checks before we do anything
+    KisResourcesSnapshotSP resources =
+            new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
+    KisNodeSP currentNode = resources->currentNode();
+    if (!currentNode || !currentNode->isEditable()) return;
+
+
+    // some layer types cannot be transformed. Give a message and return if a user tries it
+    if (currentNode->inherits("KisColorizeMask") ||
+        currentNode->inherits("KisFileLayer") ||
+        currentNode->inherits("KisCloneLayer")) {
+
+        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        kisCanvas->viewManager()->
+            showFloatingMessage(
+                i18nc("floating message in transformation tool",
+                      "Layer type cannot use the transform tool"),
+                 koIcon("object-locked"), 4000, KisFloatingMessage::High);
+
+        // force-reset transform mode to default
+        initTransformMode(mode);
+
+        return;
+    }
+
+    // this goes after the floating message check since the fetchNodesList()
+    // says a file layer is "empty" and messes up the floating message check above
+    QList<KisNodeSP> nodesList = fetchNodesList(mode, currentNode, m_workRecursively);
+    if (nodesList.isEmpty()) return;
+
+
     /**
      * We must ensure that the currently selected subtree
      * has finished all its updates.
@@ -875,67 +907,56 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
             !currentNode->paintDevice();
     }
 
-    QList<KisNodeSP> nodesList = fetchNodesList(mode, currentNode, m_workRecursively);
+    KisSelectionSP selection = resources->activeSelection();
 
-    if (nodesList.isEmpty()) {
-        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
-        kisCanvas->viewManager()->
-            showFloatingMessage(
-                i18nc("floating message in transformation tool",
-                      "Selected layer cannot be transformed with active transformation mode "),
-                 koIcon("object-locked"), 4000, KisFloatingMessage::High);
-
-        // force-reset transform mode to default
-        initTransformMode(mode);
-
-        return;
-    }
-
-
-    TransformStrokeStrategy *strategy = new TransformStrokeStrategy(currentNode, nodesList, resources->activeSelection(), image().data());
-    KisPaintDeviceSP previewDevice = strategy->previewDevice();
-
-    KisSelectionSP selection = strategy->realSelection();
-    QRect srcRect = selection ? selection->selectedExactRect() : previewDevice->exactBounds();
-
-    if (!selection && resources->activeSelection()) {
+    /**
+     * When working with transform mask, selections are not
+     * taken into account.
+     */
+    if (selection && dynamic_cast<KisTransformMask*>(currentNode.data())) {
         KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
         kisCanvas->viewManager()->
             showFloatingMessage(
                 i18nc("floating message in transformation tool",
                       "Selections are not used when editing transform masks "),
                 QIcon(), 4000, KisFloatingMessage::Low);
+
+        selection = 0;
     }
 
-    if (srcRect.isEmpty()) {
-        delete strategy;
+    TransformStrokeStrategy *strategy = new TransformStrokeStrategy(currentNode, nodesList, selection, image().data());
+    connect(strategy, SIGNAL(sigPreviewDeviceReady(KisPaintDeviceSP)), SLOT(slotPreviewDeviceGenerated(KisPaintDeviceSP)));
 
-        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
-        kisCanvas->viewManager()->
-            showFloatingMessage(
-                i18nc("floating message in transformation tool",
-                      "Cannot transform empty layer "),
-                QIcon(), 1000, KisFloatingMessage::Medium);
 
-        // force-reset transform mode to default
-        initTransformMode(mode);
+    QRect srcRect;
 
-        return;
+    if (selection) {
+        srcRect = selection->selectedExactRect();
+    } else {
+        srcRect = QRect();
+        Q_FOREACH (KisNodeSP node, nodesList) {
+            // group layers may have a projection of layers
+            // that are locked and will not be transformed
+            if (node->inherits("KisGroupLayer")) continue;
+
+            if (const KisTransformMask *mask = dynamic_cast<const KisTransformMask*>(node.data())) {
+                srcRect |= mask->sourceDataBounds();
+            } else {
+                srcRect |= node->exactBounds();
+            }
+        }
     }
 
     m_transaction = TransformTransactionProperties(srcRect, &m_currentArgs, currentNode, nodesList);
 
-    initThumbnailImage(previewDevice);
-    updateSelectionPath();
-
     if (!forceReset && fetchedFromCommand) {
         m_currentArgs = fetchedArgs;
-        initGuiAfterTransformMode();
-    } else if (forceReset || !tryInitTransformModeFromNode(currentNode)) {
-        initTransformMode(mode);
+    } else if (forceReset || !tryInitArgsFromNode(currentNode)) {
+        resetArgsForMode(mode);
     }
 
     m_strokeData = StrokeData(image()->startStroke(strategy));
+    image()->addJob(m_strokeData.strokeId(), new TransformStrokeStrategy::PreparePreviewData());
 
     bool haveInvisibleNodes = clearDevices(nodesList);
     if (haveInvisibleNodes) {
@@ -950,6 +971,8 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
 
     Q_ASSERT(m_changesTracker.isEmpty());
     commitChanges();
+
+    slotPreviewDeviceGenerated(0);
 }
 
 void KisToolTransform::endStroke()
@@ -976,6 +999,24 @@ void KisToolTransform::endStroke()
     outlineChanged();
 }
 
+void KisToolTransform::slotPreviewDeviceGenerated(KisPaintDeviceSP device)
+{
+    if (device && device->exactBounds().isEmpty()) {
+        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        kisCanvas->viewManager()->
+            showFloatingMessage(
+                i18nc("floating message in transformation tool",
+                      "Cannot transform empty layer "),
+                QIcon(), 1000, KisFloatingMessage::Medium);
+
+        cancelStroke();
+    } else {
+        initThumbnailImage(device);
+        updateSelectionPath();
+        initGuiAfterTransformMode();
+    }
+}
+
 void KisToolTransform::cancelStroke()
 {
     if (!m_strokeData.strokeId()) return;
@@ -996,11 +1037,16 @@ void KisToolTransform::commitChanges()
 {
     if (!m_strokeData.strokeId()) return;
 
-    m_changesTracker.commitConfig(m_currentArgs);
+    m_changesTracker.commitConfig(toQShared(m_currentArgs.clone()));
 }
 
-void KisToolTransform::slotTrackerChangedConfig()
+void KisToolTransform::slotTrackerChangedConfig(KisToolChangesTrackerDataSP status)
 {
+    const ToolTransformArgs *newArgs = dynamic_cast<const ToolTransformArgs*>(status.data());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(newArgs);
+
+    *m_transaction.currentConfig() = *newArgs;
+
     slotUiChangedConfig();
     updateOptionWidget();
 }
@@ -1011,7 +1057,7 @@ QList<KisNodeSP> KisToolTransform::fetchNodesList(ToolTransformArgs::TransformMo
 
     auto fetchFunc =
         [&result, mode, root] (KisNodeSP node) {
-            if (node->isEditable(node == root) &&
+        if (node->isEditable(node == root) &&
                 (!node->inherits("KisShapeLayer") || mode == ToolTransformArgs::FREE_TRANSFORM) &&
                 !node->inherits("KisFileLayer") &&
                 (!node->inherits("KisTransformMask") || node == root)) {
